


''******************************************
''*  Sea Scout Propeller-Shaft Monitor     *
''*  Author: Gregg Erickson  2011          *
''*  See MIT License for Related Copyright *
''*  See end of file and objects for .     *
''*  related copyrights and terms of use   *
''*                                        *
''*  This used code from and based on:     *
''*  Graphics Demo by Chip Gracey 2005     *
''*  TV Object by Chip Gracey 2004         *
''*  Motor_Minder by Tom Doylee 2007       *
''*  Numbers by Jeff Marten 2005           *
''*  Updated Clock by Jeff Marten 2005/201 *
''*  EEPROM -Parallax PE Kit               *
''*  Simple Serial - Chip Gracey, Phil     *
''*  Pilgrim, Jon Williams,JeffMartin 2006 *
''*  Software RTC by Jon McPhalen          *
''*  RCTIME - Andy Lindsay 2010            *
''*  Memsic 2125 Driver-Beau Schawb 2006/09*
''*  and other Parallax Inc & Forum Demos  *
''******************************************

{  This program was written to monitor and display the speed, direction and wear of twin counter rotating
propeller shafts, each driven by an 600 horsepower diesel engine on an 82 foot Point Class Cutter built in
1963 by the Coast Guard and used as a Navy missile retriever.   It also simultaneously monitors the ships pitch and
roll as well as detecting the presence of water in six watertight compartments and in drip pans under the propeller
shafts should water leak in at the shaft seals.  Temperature sensors monitor the temperature of the shaft packing seals to ensure they don't overheat due to friction from over tightening or failure to loosen under operation.

This program reads two pair of Melexis 90217 Hall sensors via a modified version of "motor_minder" to read
propeller shaft rotation and direction based upon a count of pulses and period (in milliseconds) between pulses.
The RPM calculation routine uses combined booleans of pulses detected and a timeout variable in a case selection
to deal with calculation exceptions where only the first pulse is detected, a second pulse is anticipated but
may or may not come and where it has been too long since the last pulse. In addition to rotations speed (rpm)
and pulses (revolutions), the code determines the direction of the shaft rotation by using the second hall sensor
to determine if the second hall sensor is triggered during the falling edge trigger of the first sensor which
only happens is one direction of rotation consistent with a rotary encoder configuration. A modified version
of the motor_minder object counts pulses using counters A and B and add results to objects in the main
memory and outputs in both positive and negative pulse lengths to reflect rotation direction.

The results are them displayed using the Graphics Object. Updates are written then copied to the video memory but
can be directly written to avoid memory limitations.  Each gauge can be moved around the screen by adjusting the locations in the constants.  The prop gauges have a moving needle and large numbers for RPM and the number of revolutions displayed like an odometer.  The results are also sent out via a serial port to a second prop for additional displays. And to an original mechanical gauge.

The program uses eight cogs. The first cog [0] runs the control program, bilge monitoring. Clock speed and a numbers
conversion object. The second [1] runs the TV object, the third [2] runs the graphics engine, the fourth [3] runs an attitude sensor, the fifth [4] runs a real time clock emulator, the sixth [5] monitors the port shaft monitor (modified motor minder),
the seventh [5] runs the starboard shaft monitor.   Integer math is used to reduce overhead and provide adequate accuracy.

## Issue to be resolved / tested
   # Zero Memsic, always leaves a few degrees expectial on Y
   # Calibrate Thermistors
   # Test under full loads and Check Amp Usage

Subsequent improvements could include


   * Simplify Variables
   * Re-add RPM Balance Gauge
   * Add "Ship's Listing" method with Average, Rates, etc.
   * Add Max, Min, Average for Thermisters
   * Add support for a servo to make minor adjustments in speed for balance.
   * MultiProgram Bootloading to Attitude, Clock etc..

COG and Hardware Pin Assignments are listed below:


COGs
0 Main     - Control Program, Serial, Gauge Output, Clock Speed, Bilge Monitor
1 TV       - Send Out NTSC Video Signals to TV
2 Graphics - Drawing Graphics and Put in Memory for TV
3 Realtime clock - Estimates elapsed time
4 Memsic   - Monitor X&Y Attitude through Memsic
5 Motor_M  - Monitor Port RPMs, Etc
6 Motor_M  - Monitor StarBoard RPMs, Etc


BILGE SENSORS & LED INDICATOR
( Connected to LED through 1.1 K resistor and to Input through another 1.1K resistorin series
0 Bilge - Chain Locker (Digital Input & LED Output)
1 Bilge - Forward Crew (Digital Input & LED Output)
2 Bilge - Galley (Digital Input & LED Output)
3 Bilge - Engine Room (Digital Input & LED Output)
4 Bilge - Aft Crew (Digital Input & LED Output)
5 Bilge - Lazerette (Digital Input & LED Output)

6 Bilge - Port Shaft (Digital Input & LED Output)
7 Bilge - Starboard Shaft (Digital Input & LED Output)

SERIAL COMMUNICATIONS
8 SerialPort - Receive (Rx)
9 SerialPort - Transmit(Tx)

ATTITUDE SENSOR
10 Attitude Sensor - Memsic 2125 Raw X Tilt (PWM Input)through 220 Ohm Resistor
11 Attitude Sensor - Memsic 2125 Ras Y Tilt (PWM Input)through 220 Ohm Resistor

VIDEO & SOUND
12 Video - Digital Out through 1.1K ohm resistor
13 Video - Digital Out through 560 ohm resistor
14 Video - Digital Out through 220 ohm resistor
15 Audio - (Piezo Disc)

TACHOMETER INPUT
16 Tachometer 1 - Port Speed ( Melexis 90217 Hall Sensor) through sensor protection circuit board
17 Tachometer 1 - Port Direction ( Melexis 90217 Hall Sensor)through sensor protection circuit board
18 Tachometer 2 - Starboard Speed ( Melexis 90217 Hall Sensor)through sensor protection circuit board
19 Tachometer 2 - Starboard Direction ( Melexis 90217 Hall Sensor)through sensor protection circuit board

TACHOMETER OUTPUT
20 Starboard Differential Duty Cycle Pin
21 Starboard Differential Duty Cycle Pin
22 Port Differential Duty Cycle Pin
23 Port Differential Duty Cycle Pin

THERMISTOR INPUT
24 Port 10K Thermister in parallel with 10 nF Capacitor through 100 ohm resistor
25 Starboard 10K Thermister in parallel with 10 nF Capacitor through 100 ohm resistor

UNASSIGNED PINS
26 Hibernate Signal
27 TBD

BOARD DEFAULT PINS
28 EEPROM Clock
29 EEPROM Data
30 Serial Rx - Receive
31 Serial Tx - Transmit

Vss      ground
Vdd      3.3 Volt
Vin      5 volt for melexis


 }

CON


                                      ' ---- Spin Constants --------
  _clkmode = xtal1 + pll16x           ' Set clock mode to 16x the speed of the crystal
  _xinfreq = 5_000_000                ' Define crystal speed at 5M so clockspeed is 5x16 or 80 Mhz
  _stack = ($4000 +342) >> 2          ' Set Stack large enough for display and variables


  #0, ET_MS, ET_SC, ET_MN, ET_HR, ET_DY
                                      ' -------- Constants for Video ------------------
  x_tiles = 16                        ' Define tiles for video memory used for mapping
  y_tiles = 12                        ' Define tiles for video memory used for mapping    
  paramcount = 14                     ' Define parameter count to pass to the TV object
  display_base = $5000                ' Location of video memory passed to TV


                                      '---- Constants for Video Display -----------------
                                     
  PortX  =-70                         ' Port Gauge X Location
  PortY=   37                         ' Port Gauge Y Location
  PortR=   57                         ' Port Gauge Radius, e.g. diameter
  StarX=   70                         ' Starboard Gauge X Location
  StarY=   37                         ' Starboard Gauge Y Location
  StarR=   57                         ' Starboard Gauge Radius, e.g. diameter
  TitleX=  0                          ' Title X Location
  TitleY=  91                         ' Title Y Location
  Scale=   4                          ' Number x 200 = Maximum Gauge Scale
  GaugeScale=16_777_216               ' 2?? 256


                  ' ---- Constants for Pins -------
  TxPin     = 8          'Serial Transmit Pin
  RxPin     = 9          'Serial Receive Pin
  MMx       = 10         'Memsic2125 X channel
  MMy       = 11         'Memsic2125 Y channel
  Starpin   = 16         'Encoder Pin to read Starboard Melexis 90217 - Propeller pin
  PortPin   = 18         'Encoder Pin to read Port Melexis 90217 - Propeller pin
  SGaugePinB = 20        'Output Pin to Gauge
  SGaugePinA = 21        'Output Pin to Gauge
  PGaugePinB = 22        'Output Pin to Gauge
  PGaugePinA = 23        'Output Pin to Gauge
  SThermPin  = 24        'Input Pin from Thermistor
  PThermPin  = 25        'Input Pin from Thermistor
  HibernatePin = 26      'Pin to Monitor for Hibernate Signal

                 ' ---- Constants for Program -------
  Magnets = 2            'Number of Magnets per revolution
  ExpSec  = 3           ' Seconds after pulse before counter times out,
                         ' Divide 60 by the Seconds to equal minimum RPM measured



VAR

  byte  InitX,InitY                   ' Initial value of memsic X & Y
  long  xtilt,ytilt
                                      ' ----- Pulse Variables --------------

  Long  SPeriod,    PPeriod          ' Measured Time between Pulses From Motorminder used to Calculation RPMs
  Long  SPulseCount,PPulseCount      ' Count of Hall Sensor Pulses from Motor Minder used to Calculate Revolutions
  Long  SSavedCount,PSavedCount      ' Saved Count of Pulses representing revolutions
  Long  SExpiration,PExpiration      ' Calculated Clock Count when time out will occur
  Long  SLastPulse, PLastPulse       ' Clock when last pulse came in - for calculating estimated RPM

                                      ' ------ Pulse Status Variables moved to Start Method for Faster Execution -------                                     

'(  Byte  PCaseByte,SCaseByte         ' Status Flag indicating pulse status and calculation method used
'(  Byte  PExpired, PPulseDetected    ' Boolean indicating a time , Boolean indicating if new pulse detected by Motorminder
'(  Byte  SExpired, SPulseDetected    ' Boolean indicating a time , Boolean indicating if new pulse detected by Motorminder
                                        
                                       '----Variables for Revolution Count and RPM Calculation
  
  Long  StarRPM, PortRPM              ' Calculated Starboard RPM, Port RPM accounting for pulses per rotation, etc.    
  Long  SLastRPM, PLastRPM            ' Last RPM as limit for estimated RPM
  Long  StarRevs, PortRevs            ' Calculated Revolutions of Shaft accounting for pulses per rotation, etc.
  Long  OldStarRPM,OldPortRPM         ' Saved values for clearing Screen
                                      '-------Variables for Bilge & Transmission Calculation   

  Byte  SReverse, PReverse            ' Boolean indicating if negative shaft rotation at time of pulse
  Byte  SGear,PGear                   ' Transmission Direction Status,   -1=reverse, 0=stopped, 1=forward
  Byte  SSavedGear,PSavedGear         ' Saved Transmission Direction Status,   -1=reverse, 0=stopped, 1=forward  
  Long  SShiftCount,PShiftCount       ' Count changes between neutral, forward and reverse
  Byte  Bilge_Alarm                   ' Bilge Status for up to 6 bilges [5..0] and two drip pans [7..6]


                                       '------- Variable for TV Output -------------------

  long  tv_status                      '0/1/2 = off/visible/invisible           read-only
  long  tv_enable                      '0/? = off/on                            write-only
  long  tv_pins                        '%ppmmm = pins                           write-only
  long  tv_mode                        '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen                      'pointer to screen (words)               write-only
  long  tv_colors                      'pointer to colors (longs)               write-only
  long  tv_hc                          'horizontal cells                        write-only
  long  tv_vc                          'vertical cells                          write-only
  long  tv_hx                          'horizontal cell expansion               write-only
  long  tv_vx                          'vertical cell expansion                 write-only
  long  tv_ho                          'horizontal offset                       write-only
  long  tv_vo                          'vertical offset                         write-only
  long  tv_broadcast                   'broadcast frequency (Hz)                write-only
  long  tv_auralcog                    'aural fm cog                            write-only

                                       ' --------Variables for Graphics ---------
                                       
  word  screen[x_tiles * y_tiles]      ' graphics tiles
  long  colors[64]                     ' color variables
  long  i, dx, dy                      ' index counters used to initialize graphics

                                       ' ------- Operational Variables

  byte  packet[56]                     ' Long x Revs X 2 = 8 bytes
  byte  p                              ' Long x RPM  x 2 = 8 bytes
                                       ' Long x Shifts x 2 = 8 bytes
                                       ' Alarms x 1 = 1 byte
                                         '  total 25 bytes
  byte CheckTemp                       ' Boolean Flag for temp checking & Temperature
  long Ptemp,Stemp
  byte hibernate,backupneeded          ' Power setting, Set flag to 0 for normal, -1 for hibernate
  long clockmode                       ' Variable to Set Clock Speed
  long  mytimer[5]                     ' Elapsed Timer Variable
  byte q                               ' Index for thermistor table


  long DemoStarRPM                       ' used for demo
  long DemoPortRPM                        ' used for demo
  long DemoPortStep                           'used for demo
  long demostarstep                       'used for demo

OBJ                                    ' Other Spin Objects servicing main program with 4 cogs
                                       ' The main program operates in the first cog with tv, graphics and motorminder x 2 launching new cogs

  tv    : "tv"                         ' Display Output to TV
  gr    : "graphics"                   ' Create Graphics for Display
  NuCon : "Numbers2"                   ' Create string bytes for display
  mm[2] : "Motor_Minder_Reversible"    ' Interface with Melexis 90217 and not hold up main program
  serial: "Simple_Serial"              ' Share data with second Propeller
  eeprom: "Propeller Eeprom"           ' Store and retrive variables
  clock : "UpdatedClock"               ' Adjusts speed of clock for standby
  rc    : "RCTime"                     ' Measure decay time for thermistors
  MM2125: "Memsic2125_v1.2"            ' Communicates with Memsic 2125 2 axis accelerometer
  et   :  "jm_etimer.spin"             ' Track elapsed time



PUB start |  PCaseByte,SCaseByte, PExpired, PPulseDetected, SExpired, SPulseDetected,PTherm,STherm

 
                   '-----------Start TV Object and point to the address of the TV variables ---

  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

                         '---------- initialize colors ----------------
  repeat i from 0 to 63
'
'                  Text
'                  |  Dial
'                  |  |  Needle
'                  |  |  |  Background
'                  |  |  |  |
    colors[i] := $4b_AC_06_02         ' use 3=text,2=dial,1=needle,0=background

                                 '------- Initilize Tile Screen ---------
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

                                 '------- Start Graphics,Numbers, Serial,Clock-----
gr.start                                    'Start Graphic object
gr.setup(16, 12, 128, 96, display_base)     'Setup Graphics and point to video memory used
NuCon.init                                  'Start Numbers conversion object
serial.init(RxPin,TxPin,9600)               'Start Serial Communication
clock.init(5_000_000)                       'Start clock speed object

MM2125.start(MMx, MMy)      ' ---------  Initialize Mx2125
waitcnt(clkfreq/100 + cnt)   'Wait for things to settle
MM2125.setlevel             'Assume at startup that the memsic2125 is level

InitX:=mm2125.Mx            'Record inital x
InitY:=mm2125.My            'Record inital x

                            '-------- Set Default Values and Start Motor Minders  -----

SSavedGear:=0               ' Assumed Transmission is in zero to start
PSavedGear:=0               ' Assumed Transmission is in zero to start
StarRPM:=0                  ' Assume RPM is zero
PortRPM:=0                  ' Assume RPM is zero
SExpired:=true              ' Assume no recent pulses when starting
PExpired:=true              ' Assume no recent pulses when starting

                 ' ------Read stored counts and launch MotorMinders ----------

eeprom.VarRestore(@SShiftCount,@SShiftCount+3) ' Read previous shift count from EEPROM
eeprom.VarRestore(@PShiftCount,@PShiftCount+3) ' Read previous shift count from EEPROM
eeprom.VarRestore(@SPulseCount,@SPulseCount+3) ' Read previous pulse count from EEPROM
eeprom.VarRestore(@PPulseCount,@PPulseCount+3) ' Read previous pulse count from EEPROM

eeprom.VarRestore(@mytimer,@mytimer+20)         ' Store Rundays to  EEPROM

SSavedCount:=SPulseCount                       ' Assume saved pulse count equals count
PSavedCount:=PPulseCount                       ' Assume saved pulse count equals count

et.init(-1)                                  ' Start Elapsed Time and load memory
et.write(@mytimer)


mm[0].Start(Portpin, @PPeriod, @PPulseCount)   ' Launch Port Motor Minder
mm[1].Start(Starpin, @SPeriod, @SPulseCount)   ' Launch Starboard Motor Minder


                  '--
                        ' Setup Starboard Gauge
frqa :=128*gaugescale         ' Set ctra duty cycle default
ctra[30..26] := %00111        ' Set ctra to differential dutymode
ctra[14..9] := SGaugePinB     ' Set ctra's BPIN
ctra[5..0] := SGaugePinA      ' Set ctra's APIN
dira[SGaugePinB]~~            ' Set Gauge Pin to output
dira[SGaugePinA]~~            ' Set Gauge Pin to output

                        ' Setup Port Gauge
frqb :=128*gaugescale         ' Set ctrb duty cycle default
ctrb[30..26] := %00111        ' Set ctrb to differential dutymode
ctrb[14..9] := PGaugePinB     ' Set ctrb's BPIN
ctrb[5..0] := PGaugePinA      ' Set ctrb's APIN
dira[PGaugePinB]~~            ' Set Gauge Pin to output
dira[PGaugePinA]~~            ' Set Gauge Pin to output


    '-----------Draw Static Titles ---------------------
    'draw title box
    gr.textmode(1,1,6,5)          'Set text properties
    gr.colorwidth(2,0)            'Set text color and width
    gr.text(TitleX,TitleY,@pitch) 'Write Pitch Label
    gr.text(TitleX,-40,@roll)     'Write Roll Label


    '----------Draw Static Part of Gauges ------------------------------
    '(note: 1 degree is about 23 ticks for angle calculation ) ----------

    gr.color(2)
    gr.arc(PortX,PortY,PortR,PortR,0,23,360,0)           'Draw Port Dial Dial
    gr.arc(StarX,StarY,StarR,StarR,0,23,360,0)           'Draw Starboard Dial
    repeat i from -100 to 200 step 10
      gr.arc(PortX,PortY,PortR,PortR,23*i,23,1,0)        'Draw Port Rotational Ticks
      gr.arc(PortX,PortY,PortR-5,PortR-5,23*i,23,1,1)
      gr.arc(StarX,StarY,StarR,StarR,23*i,23,1,0)        'Draw Starbard Rotational Ticks
      gr.arc(StarX,StarY,StarR-5,StarR-5,23*i,23,1,1)
      if i//50==0
        gr.arc(PortX,PortY,PortR,PortR,23*i,23,1,0)      'Draw Port Large Ticks
        gr.arc(PortX,PortY,PortR-10,PortR-10,23*i,23,1,1)
        gr.arc(StarX,StarY,StarR,StarR,23*i,23,1,0)      'Draw Starboard Large Tick
        gr.arc(StarX,StarY,StarR-10,StarR-10,23*i,23,1,1)


 Dira[HibernatePin]~    'Set Pin 25 to Input for Hibernate Signal

 demoportstep:=10          ' used for demo
 demostarstep:=10          'used for demo
 demoportrpm:=-250
 demostarrpm:=50

 CheckTemp:=true        'Set Temperature Check to True
 
Repeat          '--------- Run Main Control Loop ----------



      If Ina[HibernatePin] ' ----If the Hibernate *in is high then Slow and Wait ------
           Hibernate:=true
           Backupneeded:=true

     If Backupneeded
           BackupNeeded:=false
           eeprom.VarBackup(@SShiftCount,@SShiftCount+3) ' Store Variable to  EEPROM
           eeprom.VarBackup(@PShiftCount,@PShiftCount+3) ' Store Variable to  EEPROM
           eeprom.VarBackup(@SPulseCount,@SPulseCount+3) ' Store Variable to  EEPROM
           eeprom.VarBackup(@PPulseCount,@PPulseCount+3) ' Store Variable to  EEPROM

           eeprom.VarBackup(@Mytimer,@Mytimer+20)        ' Store Rundays to  EEPROM

         
     If Hibernate
           Hibernate:=false               ' Clear Hibernate Flag

           tv.stop
           gr.stop
           MM2125.stop
           et.finalize
           mm[0].stop
           mm[1].stop



           Repeat p from 1 to 7           ' Stop unecessary cogs to save power
                Cogstop(p)
           Clock.SetClock(rcslow)         ' Slow clock to RC Slow ~ 20Khz to save power


           waitpne(%00000000,%11111111,0)     ' Wait till any pin [0..7] go high indicating bilge

           Clock.SetClock(xtal1 + pll16x)     ' Reset clock to full speed = 80 mhz
           tv.start(@tv_status)                              ' Restart TV object
           gr.start                                          ' Restart graphic object
           gr.setup(16, 12, 128, 96, display_base)           ' Reset graphics and point to video memory used


           MM2125.start(MMx, MMy)                            ' Restart Memsic
           waitcnt(clkfreq/10 + cnt)                         ' Wait for things to settle
           MM2125.setlevel                                   ' Calibrate as Level
           et.init(-1)                                      ' Start Elapsed Time and load memory
           et.write(@mytimer)



           mm[0].Start(Portpin, @PPeriod, @PPulseCount)      ' Relaunch Port Motor Minder
           mm[1].Start(Starpin, @SPeriod, @SPulseCount)      ' Relaunch Launch Starboard Motor Minder
           pexpired:=true
           sexpired:=true


    ' ---------------- Check Bilges, Temp, Runtime, RPM  ------


    rc.rctime(SThermPin, 1, @STherm)    ' Measure P24 resistance via RCTIME to Tell Temp
    rc.rctime(PThermPin, 1, @PTherm)    ' Measure P25 resistance via RCTIME to Tell Temp

    bilge_alarm:=check_bilges             ' Check Bilges, turn on tone & LED if alarm else show heartbeat on LEDs

     '---------------- Check Runtime -----------
    

    et.read(@mytimer)

     '-------------- Package and Transmit Serial Packets to Other Locations
    xtilt:=mm2125.mxTilt-6
    ytilt:=mm2125.myTilt
    packet[0]:=bilge_alarm               ' Load byte to serial packet      
    repeat p from 0 to 3                 ' Load longs(4x bytes) to serialpacket
      packet[p+1]:=byte[@starrpm][p]
      packet[p+5]:=byte[@portrpm][p]
      packet[p+9]:=byte[@starrevs][p]
      packet[p+13]:=byte[@portrevs][p]'
      packet[p+17]:=byte[@sshiftcount][p]
      packet[p+21]:=byte[@pshiftcount][p]
      packet[p+25]:=byte[@mytimer[4]][p]
      packet[p+29]:=byte[@mytimer[3]][p]
      packet[p+33]:=byte[@mytimer[2]][p]
      packet[p+37]:=byte[@mytimer[1]][p]
      packet[p+43]:=byte[@Xtilt][p]
      packet[p+47]:=byte[@YTilt][p]



    repeat q from 0 to 100
        if (309*stherm/10)>word[@Rtable][q]
            stemp:=(180*q/100)+32
            quit
        else
            stemp:=999
    repeat q from 0 to 100
        if (309*Ptherm/10)>word[@Rtable][q]
            ptemp:=(180*q/100)+32
            quit
        else
            ptemp:=999
    '


    packet[41]:=STemp
    packet[42]:=PTemp



    
    repeat  p from 0 to 50               ' Transmit serial packets
       serial.tx(packet[p])              ' 0-43 bytes sent
    serial.tx(10)                        ' Send linefeed  , byte 44 sent
    serial.tx(13)                        ' Send newline   , byte 45 sent


     '------------   Calculate Port RPM and Revolutions ---------------------

    PReverse:=(PPeriod<0)
    PPulseDetected:=(PPulseCount<>PSavedCount)

    PCasebyte~                                    ' clear byte used for case selection
    if PPulseDetected
          PCasebyte:=PCasebyte + %00000010        ' add two a.k.a. set second bit if plse detected
    if PExpired
          PCasebyte:=PCasebyte + %00000001        ' add one a.k.a. set first bit if expired or timed out

    Case  PCasebyte                                'use case to determine appropriate calculatiion
    {  3:                'Respond to Trigger or First Pulse  "First"
          PortRPM:=0.0   ' This is not needed if motorminder syncs on first pulses
          PLastRPM:=PortRPM
          PSavedCount:=PPulseCount
          PExpired:=false
          PExpiration:=((ExpSec*clkfreq)+cnt)
    }
                          'Respond to Second Pulse or On-Going Pulses   "Pulse"
      3,2:
          PLastPulse:=cnt
          PortRPM:=60000/Pperiod/magnets
          PLastRPM:=PortRPM
          PSavedCount:=PPulseCount
          PExpired:=false
          PExpiration:=((ExpSec*clkfreq)+cnt)

      0:               'Respond when Waiting for Pulses "Waiting"
          PPeriod:= ((cnt-PLastPulse)  / (clkfreq / 1_000_000) + 1) / 1000
          PortRPM:=60000/Pperiod/magnets

          If (||PortRPM)>(||PLastRPM)
             PortRPM:=PLastRPM
          If Cnt>PExpiration
             PExpired:=true

          If PReverse
             PortRPM:=-1*(||PortRPM)
             PPeriod:=-1*PPeriod

      1:                   'Respond when Expired or Time Out "Time Out"
          PortRPM:=0.0
          PLastRPM:=PortRPM

                     ' -------- Check Port Gears -----------------

    If PortRPM>0      ' Check Port Gear, 2=reverse, 0=neutral, 1=forward
       PGear:=1
    elseif PortRPM<0
       PGear:=2
    else
       PGear:=0
    If not (PGear==PSavedGear)      ' Compare to Previous Gear Setting
      PShiftCount:=PShiftCount+1    ' Increment Shift Counter if Changed
      PsavedGear:=PGear             ' Store New Gear Setting if Changed
       if PGear==0
         BackupNeeded:=true

       '------------  Calculate Starboard RPM and Revolutions ---------------------

    SReverse:=(SPeriod<0)
    SPulseDetected:=(SPulseCount<>SSavedCount)
    SCasebyte~                                            ' clear byte used for case selection
      if SPulseDetected
          SCasebyte:=SCasebyte + %00000010                 ' add two a.k.a. set second bit if starboard pulse detected
      if SExpired
          SCasebyte:=SCasebyte + %00000001                  'add one a.k.a. set first bit if starbboard pulse expired

     Case SCasebyte                                         ' use bits in simple case selection

     {   3:   'Respond to Trigger or First Pulse  "First"
          StarRPM:=0.0   ' This is not needed if motorminder syncs on first pules
          SLastRPM:=StarRPM
          SSavedCount:=SPulseCount
          SExpired:=false
          SExpiration:=((ExpSec*clkfreq)+cnt)
      }
        3,2:  'Respond to Second Pulse or On-Going Pulses   "Pulse"
          SLastPulse:=cnt
          StarRPM:=60000/Speriod/magnets
          SLastRPM:=StarRPM
          SSavedCount:=SPulseCount
          SExpired:=false
          SExpiration:=((ExpSec*clkfreq)+cnt)

        0:   'Respond when Waiting for Pulses "Waiting"
          SPeriod:= ((cnt-SLastPulse)  / (clkfreq / 1_000_000) + 1) / 1000
          StarRPM:=60000/Speriod/magnets
          If (||StarRPM)>(||SLastRPM)
             StarRPM:=SLastRPM
          If Cnt>SExpiration
             SExpired:=true
          If SReverse
             StarRPM:=-1*(||StarRPM)
             SPeriod:=-1*SPeriod

       1:   'Respond when Expired or Time Out "Time Out"
          StarRPM:=0.0
          SLastRPM:=StarRPM

 
  demoStarRPM:=demoStarRPM+demostarstep
  if (DemoStarRPM>600)
     Demostarstep:=-10
  if DemoStarRPM < -600
     Demostarstep:=10
     
     
  demoportRPM:=demoportRPM+demoportstep
  if DemoPortRPM>600
     Demoportstep:=-10
  if DemoPortRPM< -600
     Demoportstep:=10
    

  
  starrpm:=demostarrpm
  portrpm:=demoportrpm



                   ' -------- Check Starboard Gears -----------------

    If StarRPM>0    ' Check Starboard Gear,2=reverse,0=neutral,1=forward
       SGear:=1
    elseif StarRPM<0
       SGear:=2
    else
       SGear:=0
    If not (SGear==SSavedGear)      ' Compare to Previous Gear Setting
      SShiftCount:=SShiftCount+1    ' Increment Shift Counter if changed
      SsavedGear:=SGear             ' Store New Gear Setting if Changed  
      if SGear==0
         BackupNeeded:=true

    '-----Convert for magnets on sensor before display

    Starrevs:=Spulsecount/magnets         ' Move Pulsecount to Revolutions Count.  Make multiple magnet adjustment here
    Portrevs:=Ppulsecount/magnets         ' Move Pulsecount to Revolutions Count.  Make multiple magnet adjustment here




    '-----Clear Old Needle and Numbers with Background Color---

    gr.color(0)
    gr.box(starx-30,starY+10,50,30)  ' Clear RPM Numbers
    gr.box(Portx-30,PortY+10,50,30)

    gr.box(starx-25,starY-25,57,10)  ' Clear Revolutin Odometer
    gr.box(Portx-25,PortY-25,57,10)

                                     ' Clear Shift Count
    gr.box(starx-25,starY-45,50,20)
    gr.box(Portx-25,PortY-45,50,20)

   
    gr.colorwidth(0,0)               ' Clear Needle
    gr.arc(PortX,PortY,PortR-16,PortR-16,23*(200-(||OldPortRPM)/2),23,4,3)      'Draw Starboard Needle
    gr.arc(StarX,StarY,StarR-16,StarR-16,23*(200-(||OldStarRPM)/2),23,4,3)
    OldStarRPM:=StarRPM
    OldPortRPM:=PortRPM

   
  

    '------ Draw Needles-----


    if PortRPM>0
        gr.colorwidth(1,0)
    else
        gr.colorwidth(2,0)                                                      'Set Gauge Color and Thickness
    gr.arc(PortX,PortY,PortR-16,PortR-16,23*(200-(||PortRPM)/2),23,4,3)      'Draw Starboard Needle
    if StarRPM>0
        gr.colorwidth(1,0)
    else
        gr.colorwidth(2,0)

    gr.arc(StarX,StarY,StarR-16,StarR-16,23*(200-(||StarRPM)/2),23,4,3)

     '---- Gauge Pivot Pins ---
    
       gr.colorwidth(1,12)            'Thick pivot pin like real gauge
       gr.plot(PortX,PortY)          'Draw Port Pivot
       gr.plot(StarX,StarY)          'Draw Starboard Pivot
         
  
    '---- Gauge RPM Numbers ---

    if StarRPM>0
        gr.colorwidth(2,0)
    else
        gr.colorwidth(1,0)
    gr.textmode(2,3,6,5)
    gr.text(StarX-5,StarY+25,Nucon.ToStr(StarRPM,%000_000_000_0_0_000100_01010)) 'Convert and Post Starboard Revolutions
    gr.finish
                                                            'Set Gauge Color and Thickness
    if PortRPM>0
        gr.colorwidth(2,0)
    else
        gr.colorwidth(1,0)
    gr.text(PortX-5,PortY+25,Nucon.ToStr(PortRPM,%000_000_000_0_0_000100_01010)) 'Convert and Post Starboard Revolutions
    gr.finish

    ' '--- Counter and Gear Displays---


    '  Note--> revolutions are divided by 3.5 due to hardware induced glitch caused by the protection hardware not switching fast enough
    gr.textmode(1,1,6,5)
    gr.colorwidth(2,0)
    gr.text(StarX,StarY-20,Nucon.ToStr(2*StarRevs/7,%000_000_000_0_0_001010_01010)) 'Convert and Post Starboard Revolutions
    gr.finish                                                                 'Finish graphic write beore next graphic
    gr.text(PortX,PortY-20,Nucon.ToStr(2*PortRevs/7,%000_000_000_0_0_001010_01010))  'Convert and Post Starboard Revolutions
    gr.finish       'Finish graphic write beore next graphic


    gr.color(0)

    case SGear        ' Display "R" for Reverse, "N" for Neutral or "F" for Forward Below Rev Count
    
      2:
         gr.text(StarX,StarY,@R)

      0:
         gr.text(StarX,StarY,@N)

      1:
         gr.text(StarX,StarY,@F)

    case PGear
    
      2:
         gr.text(PortX,PortY,@R)

      0:
         gr.text(PortX,PortY,@N)

      1:
         gr.text(PortX,PortY,@F)

    gr.color(2)
    gr.textmode(1,1,6,5)   
  
    gr.text(StarX-2,StarY-30,Nucon.ToStr(SShiftCount,%1010)) 'Convert and Post Starboard Shift Count
    gr.finish                                                                 'Finish graphic write beore next graphic
    gr.text(PortX-2,PortY-30,Nucon.ToStr(PShiftCount,%1010))  'Convert and Post Port Shift Count
    gr.finish                                                          'Finish graphic write beore next graphic
      ' ---------- Post Temp to Screen ------------------


    gr.color(0)
    gr.box(starx-25,starY-75,50,12)
    gr.box(Portx-25,PortY-75,50,12)  
    gr.color(2)
    gr.textmode(1,1,6,9)
    gr.text(StarX,StarY-40,Nucon.ToStr(Stemp,10)) 'Convert and Post Starboard Temperatures
    gr.finish                                                                 'Finish graphic write beore next graphic
    gr.text(PortX,PortY-40,Nucon.ToStr(PTemp,10))  'Convert and Post Port Temperature
    gr.finish
    gr.textmode(1,1,6,5)
    gr.text(StarX+5,StarY-40,@deg)
    gr.text(PortX+5,PortY-40,@deg)
    
    '-     --------- Post Tilt ---------


   { Plot X tilt or pitch MM2125.MxTilt - Modify Scalar with sensor on}

    gr.color(0)
    gr.box(-9,-35,19,115)
    gr.colorwidth(1,30)
    gr.plot(0,(30-MM2125.MxTilt))
    gr.colorwidth(0,0)        
    gr.plot(0,80)
    gr.colorwidth(1,2)
    gr.line(0,-30)
    gr.colorwidth(0,0)
    gr.text(-3,(30-MM2125.MxTilt),nucon.tostr(||(mm2125.MxTilt-6),%1010))

   
   { Plot Y tilt or roll MM2125.MyTilt - Modify Scalar with sensor on}
    gr.color(0)
    gr.box(-100,-65,205,20)
    gr.colorwidth(1,30)
    gr.plot((MM2125.MyTilt),-55)
    gr.colorwidth(0,0)     
    gr.plot(-90,-55)
    gr.colorwidth(1,2)
    gr.line(90,-55)    
    gr.colorwidth(0,0)
    gr.text((MM2125.MyTilt)-3,-55,nucon.tostr(||(mm2125.MyTilt),%1010))


 

'     ' ----------------- Post Output to Gauges ----------------------

    frqa := (255*(740-starrpm)/(740*2)) * gaugescale    ' calculate starboard duty cycle and update frqa register
    frqb := (255*(740-Portrpm)/(740*2)) * gaugescale    ' calculate zport duty cycle and update frqb register

 '-------------End of Main Calculation and Display Loop ------------------

  
Pub test_bilge_leds | bilgenum  ' ---- Check LEDs-----

dira[0..7]~~                   'Set bilge pins to input

repeat bilgenum from 0 to 7    'Cycle through 8 pins
     outa[bilgenum]~~          'Turn LED on
     waitcnt((clkfreq/1000#> 381)+cnt) 'Wait a moment
     outa[bilgenum]~           'Turn LED Off


PUB check_bilges:BA | j         ' ----- Check Bilges-----

test_bilge_leds

dira[0..7]~                     ' Set Pins to Input
BA:=ina[7..0]                   ' Get Status of Pins


if BA>0
        gr.color(0)
        gr.box(-130,-95,240,20)
        gr.colorwidth(1,0)
        gr.text(-95,-85,@Alarm)
        repeat j from 0 to 7    ' Post bilge #s with high pin
           if ina[j]==1
             
             gr.colorwidth(1,15)
             gr.plot((-50+(j*15)),-85)
             gr.textmode(1,1,6,5)
             gr.colorwidth(0,0)
             gr.text((-50+(j*15)),-85,nucon.tostr(j,%1010))
           
else
           gr.color(0)
           gr.box(-130,-95,240,20)
           gr.colorwidth(2,0)
           gr.textmode(1,1,6,5)
           gr.text(-35,-85,@bilge)
           gr.text(53,-85,nucon.tostr(mytimer[4],%1010))
           gr.finish
           gr.text(68,-85,nucon.tostr(mytimer[3],%1010))
           gr.finish
           gr.text(83,-85,nucon.tostr(mytimer[2],%1010))
           gr.finish
           gr.text(98,-85,nucon.tostr(mytimer[1],%1010))

result:=ina[7..0]               ' Return Results for use elsewhere







DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog

ptitle                  byte    "Shaft",0               'text
psubtitle               byte    "RPM",0                 'text
pitch                   byte    "Pitch",0         'text
roll                    byte    "Roll",0         'text
alarm                   byte    "Bilge Alarm",0  'text
port                    byte    "P",0                'text
star                    byte    "S",0           'text 
bilge                   byte    "No Bilge Alarms, Runtime=",0     'text
pulse                   byte    "Pulse",0               'text
Comma                   byte    ",",0                   'text
F                       byte    "F",0                   'text
N                       byte    "N",0                   'text
R                       byte    "R",0                   'text
Deg                     byte    $B0,"F",0
RTable                  word    31770,30250, 28820, 27450, 26160
                        word    24940, 23770, 22670, 21620, 20630
                        word    19680, 18780, 17930, 17120, 16350
                        word    15620, 14930, 14260, 13630, 13040
                        word    12470, 11920, 11410, 10910, 10450
                        word    10000, 9575, 9170, 8784, 8416
                        word    8064, 7730, 7410, 7106, 6815
                        word    6538, 6273, 6020, 5778, 5548
                        word    5327, 5117, 4915, 4723, 4539
                        word    4363, 4195, 4034, 3880, 3733
                        word    3592, 3457, 3328, 3204, 3086
                        word    2972, 2863, 2759, 2659, 2564
                        word    2472, 2384, 2299, 2218, 2141
                        word    2066, 1994, 1926, 1860, 1796
                        word    1735, 1677, 1621, 1567, 1515
                        word    1465, 1417, 1371, 1326, 1284
                        word    1243, 1203, 1165, 1128, 1093
                        word    1059, 1027, 995, 965, 936
                        word    908, 881, 855, 829, 805
                        word    782, 759, 737, 715, 695