'' =================================================================================================
''
''   File....... jm_etimer.spin
''   Purpose.... Background timer
''   Author..... Jon "JonnyMac" McPhalen (aka Jon Williams)
''               Copyright (c) 2009 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 21 JUL 2009
''   Updated.... 21 JUL 2009
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  
  
con

  #0, ET_MS, ET_SC, ET_MN, ET_HR, ET_DY
  

obj


var

  long  cog

  long  timermode                                               ' 0 = hold, else run
  long  timer[5]                                                ' event timer


pub init(p) | okay

'' Starts event timer driver
'' -- p is "heartbeat" pin (use -1 to disable)

  finalize                                                      ' stop if already running 

  MS_001 := clkfreq / 1_000                                     ' cnt ticks per millisecond

  if (p => 0) and (p =< 27)                                     ' if valid pin (protect rx, tx, i2c)
    hbmask := 1 << p                                            '   create mask

  mdpntr := @timermode
  mspntr := @timer[0]
  scpntr := @timer[1]  
  mnpntr := @timer[2]  
  hrpntr := @timer[3]  
  dypntr := @timer[4]

  clear                                                         ' clear registers
  resume                                                        ' enable timer
   
  okay := cog := cognew(@etimer, 0) + 1                         ' start timer cog

  return okay


pub finalize

'' Stops event timer; frees a cog 

  if cog
    cogstop(cog~ - 1)
    clear   


pub read(pntr)

'' Copies timer registers to array at pntr

  longmove(pntr, @timer, 5)  


pub write(pntr)

'' Copies values in array at pntr to timer registers

  longmove(@timer, pntr, 5)


pub set(_ms, _sc, _mn, _hr, _dy)

'' Set individual clock registers

  longmove(@timer, @_ms, 5)
  
  
pub readreg(reg) | value

'' Reads selected register from the timer

  if (reg => ET_MS) and (reg =< ET_DY)
    value := etimer[reg]
  else
    value := -1

  return value

    
pub writereg(reg, value) | okay

'' Returns selected register from the timer

  if (reg => ET_MS) and (reg =< ET_DY) 
    etimer[reg] := value
    okay := true
  else
    okay := false

  return okay    


pub hold

'' Stops timer; does not clear

  mdpntr := false


pub resume

'' Restarts timer; does not clear 

  mdpntr := true


pub clear

'' Clears timer value; does not stop timer

  longfill(@timer, 0, 5)


dat

                        org     0

etimer                  mov     ms1timer, MS_001                        ' initialize timer
                        add     ms1timer, cnt                           ' sync
                        tjz     hbmask, #checkmode                      ' hb enabled?
                        andn    outa, hbmask                            ' yes, clear pin
                        or      dira, hbmask                            ' make output 

checkmode               rdlong  tmp1, mdpntr            wz              ' get timer mode
                if_z    jmp     #etimer                                 ' if 0, disable
                
                        waitcnt ms1timer, MS_001                        ' else wait on 1ms timer
  
getms                   rdlong  tmp1, mspntr                            ' read current ms

heartbeat               tjz     hbmask, #updatems                       ' skip if no hb  
                        cmp     tmp1, #0                wz              ' if 0
                if_e    xor     outa, hbmask                            '   toggle
                if_e    jmp     #updatems
                        cmp     tmp1, #500              wz              ' or 500
                if_e    xor     outa, hbmask                            '   toggle

updatems                add     tmp1, #1                                ' increment
                        cmp     tmp1, SEC_01            wz              ' at limit?
                if_e    mov     tmp1, #0                                ' if yes, clear
                        wrlong  tmp1, mspntr                            ' write back new value
                if_ne   jmp     #checkmode                              ' to top if no roll-over
                
updatesecs              rdlong  tmp1, scpntr
                        add     tmp1, #1
                        cmp     tmp1, #60               wz
                if_e    mov     tmp1, #0
                        wrlong  tmp1, scpntr
                if_ne   jmp     #checkmode                            

updatemins              rdlong  tmp1, mnpntr
                        add     tmp1, #1
                        cmp     tmp1, #60               wz
                if_e    mov     tmp1, #0
                        wrlong  tmp1, mnpntr
                if_ne   jmp     #checkmode

updatehrs               rdlong  tmp1, hrpntr
                        add     tmp1, #1
                        cmp     tmp1, #24               wz
                if_e    mov     tmp1, #0
                        wrlong  tmp1, hrpntr
                if_ne   jmp     #checkmode

updatedys               rdlong  tmp1, dypntr
                        add     tmp1, #1
                        cmp     tmp1, #365              wz
                if_e    mov     tmp1, #0
                        wrlong  tmp1, dypntr
                if_ne   jmp     #checkmode
                
' --------------------------------------------------------------------------------------------------

SEC_01                  long    1_000

MS_001                  long    0-0                             ' cnt ticks per millisecond

hbmask                  long    0-0                             ' heartbeat pin mask

mdpntr                  long    0-0                             ' hub address of timer mode
mspntr                  long    0-0                             ' hub address of milliseconds
scpntr                  long    0-0                             ' hub address of seconds
mnpntr                  long    0-0                             ' hub address of minutes
hrpntr                  long    0-0                             ' hub address of hours
dypntr                  long    0-0                             ' hub address of days

ms1timer                res     1                               ' for 1ms timing
tmp1                    res     1
tmp2                    res     1

                        fit     492
                        

dat

{{

  Copyright (c) 2009 Jon McPhalen (aka Jon Williams)

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}                   
